@@ -1,7 +1,7 @@
-/* A Bison parser, made by GNU Bison 1.875a.  */
+/* A Bison parser, made by GNU Bison 1.875d.  */
 
 /* Skeleton parser for Yacc-like parsing with Bison,
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -61,13 +61,19 @@
      STARTTLS = 263,
      CERTIFICATE_PASSPHRASE = 264,
      PRECONNECT = 265,
-     MDA = 266,
-     MAP = 267,
-     DISABLED = 268,
-     ENABLED = 269,
-     REQUIRED = 270,
-     STRING = 271,
-     NUMBER = 272
+     POSTCONNECT = 266,
+     MDA = 267,
+     QUALIFYDOMAIN = 268,
+     HELO = 269,
+     FORCE = 270,
+     SENDER = 271,
+     REVERSE_PATH = 272,
+     MAP = 273,
+     DISABLED = 274,
+     ENABLED = 275,
+     REQUIRED = 276,
+     STRING = 277,
+     NUMBER = 278
    };
 #endif
 #define IDENTITY 258
@@ -78,13 +84,19 @@
 #define STARTTLS 263
 #define CERTIFICATE_PASSPHRASE 264
 #define PRECONNECT 265
-#define MDA 266
-#define MAP 267
-#define DISABLED 268
-#define ENABLED 269
-#define REQUIRED 270
-#define STRING 271
-#define NUMBER 272
+#define POSTCONNECT 266
+#define MDA 267
+#define QUALIFYDOMAIN 268
+#define HELO 269
+#define FORCE 270
+#define SENDER 271
+#define REVERSE_PATH 272
+#define MAP 273
+#define DISABLED 274
+#define ENABLED 275
+#define REQUIRED 276
+#define STRING 277
+#define NUMBER 278
 
 
 
@@ -100,10 +112,18 @@
  */
 
 
+#include "config.h"
+
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
 
 #include <libesmtp.h>
 
@@ -112,6 +132,8 @@
 #include "local.h"
 #include "xmalloc.h"
 
+extern int yylex (void);
+
 
 /** Path name of dot file */
 static const char *rcfile = NULL;		
@@ -156,13 +178,13 @@ void yyerror (const char *s);
 #endif
 
 #if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
-#line 52 "parser.y"
+#line 62 "parser.y"
 typedef union YYSTYPE {
     int number;
     char *sval;
 } YYSTYPE;
 /* Line 191 of yacc.c.  */
-#line 166 "parser.c"
+#line 188 "parser.c"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
@@ -174,22 +196,29 @@ typedef union YYSTYPE {
 
 
 /* Line 214 of yacc.c.  */
-#line 178 "parser.c"
+#line 200 "parser.c"
 
 #if ! defined (yyoverflow) || YYERROR_VERBOSE
 
+# ifndef YYFREE
+#  define YYFREE free
+# endif
+# ifndef YYMALLOC
+#  define YYMALLOC malloc
+# endif
+
 /* The parser invokes alloca or malloc; define the necessary symbols.  */
 
-# if YYSTACK_USE_ALLOCA
-#  define YYSTACK_ALLOC alloca
+# ifdef YYSTACK_USE_ALLOCA
+#  if YYSTACK_USE_ALLOCA
+#   define YYSTACK_ALLOC alloca
+#  endif
 # else
-#  ifndef YYSTACK_USE_ALLOCA
-#   if defined (alloca) || defined (_ALLOCA_H)
-#    define YYSTACK_ALLOC alloca
-#   else
-#    ifdef __GNUC__
-#     define YYSTACK_ALLOC __builtin_alloca
-#    endif
+#  if defined (alloca) || defined (_ALLOCA_H)
+#   define YYSTACK_ALLOC alloca
+#  else
+#   ifdef __GNUC__
+#    define YYSTACK_ALLOC __builtin_alloca
 #   endif
 #  endif
 # endif
@@ -202,20 +231,20 @@ typedef union YYSTYPE {
 #   include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
 #   define YYSIZE_T size_t
 #  endif
-#  define YYSTACK_ALLOC malloc
-#  define YYSTACK_FREE free
+#  define YYSTACK_ALLOC YYMALLOC
+#  define YYSTACK_FREE YYFREE
 # endif
 #endif /* ! defined (yyoverflow) || YYERROR_VERBOSE */
 
 
 #if (! defined (yyoverflow) \
      && (! defined (__cplusplus) \
-	 || (YYSTYPE_IS_TRIVIAL)))
+	 || (defined (YYSTYPE_IS_TRIVIAL) && YYSTYPE_IS_TRIVIAL)))
 
 /* A type that is properly aligned for any stack member.  */
 union yyalloc
 {
-  short yyss;
+  short int yyss;
   YYSTYPE yyvs;
   };
 
@@ -225,13 +254,13 @@ union yyalloc
 /* The size of an array large to enough to hold all stacks, each with
    N elements.  */
 # define YYSTACK_BYTES(N) \
-     ((N) * (sizeof (short) + sizeof (YYSTYPE))				\
+     ((N) * (sizeof (short int) + sizeof (YYSTYPE))			\
       + YYSTACK_GAP_MAXIMUM)
 
 /* Copy COUNT objects from FROM to TO.  The source and destination do
    not overlap.  */
 # ifndef YYCOPY
-#  if 1 < __GNUC__
+#  if defined (__GNUC__) && 1 < __GNUC__
 #   define YYCOPY(To, From, Count) \
       __builtin_memcpy (To, From, (Count) * sizeof (*(From)))
 #  else
@@ -267,26 +296,26 @@ union yyalloc
 #if defined (__STDC__) || defined (__cplusplus)
    typedef signed char yysigned_char;
 #else
-   typedef short yysigned_char;
+   typedef short int yysigned_char;
 #endif
 
 /* YYFINAL -- State number of the termination state. */
-#define YYFINAL  24
+#define YYFINAL  33
 /* YYLAST -- Last index in YYTABLE.  */
-#define YYLAST   46
+#define YYLAST   64
 
 /* YYNTOKENS -- Number of terminals. */
-#define YYNTOKENS  18
+#define YYNTOKENS  24
 /* YYNNTS -- Number of nonterminals. */
 #define YYNNTS  7
 /* YYNRULES -- Number of rules. */
-#define YYNRULES  23
+#define YYNRULES  28
 /* YYNRULES -- Number of states. */
-#define YYNSTATES  40
+#define YYNSTATES  56
 
 /* YYTRANSLATE(YYLEX) -- Bison symbol number corresponding to YYLEX.  */
 #define YYUNDEFTOK  2
-#define YYMAXUTOK   272
+#define YYMAXUTOK   278
 
 #define YYTRANSLATE(YYX) 						\
   ((unsigned int) (YYX) <= YYMAXUTOK ? yytranslate[YYX] : YYUNDEFTOK)
@@ -321,7 +350,7 @@ static const unsigned char yytranslate[] =
        2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
        2,     2,     2,     2,     2,     2,     1,     2,     3,     4,
        5,     6,     7,     8,     9,    10,    11,    12,    13,    14,
-      15,    16,    17
+      15,    16,    17,    18,    19,    20,    21,    22,    23
 };
 
 #if YYDEBUG
@@ -331,27 +360,30 @@ static const unsigned char yyprhs[] =
 {
        0,     0,     3,     4,     6,     9,    11,    13,    16,    20,
       21,    23,    27,    29,    32,    36,    40,    44,    48,    52,
-      56,    60,    64,    68
+      56,    60,    64,    68,    72,    76,    81,    86,    90
 };
 
 /* YYRHS -- A `-1'-separated list of the rules' RHS. */
 static const yysigned_char yyrhs[] =
 {
-      19,     0,    -1,    -1,    23,    -1,    23,    20,    -1,    20,
-      -1,    22,    -1,    22,    23,    -1,    20,    22,    23,    -1,
-      -1,    12,    -1,     3,    21,    16,    -1,    24,    -1,    23,
-      24,    -1,     5,    21,    16,    -1,     6,    21,    16,    -1,
-       7,    21,    16,    -1,     8,    21,    13,    -1,     8,    21,
-      14,    -1,     8,    21,    15,    -1,     9,    21,    16,    -1,
-      10,    21,    16,    -1,    11,    21,    16,    -1,     4,    -1
+      25,     0,    -1,    -1,    29,    -1,    29,    26,    -1,    26,
+      -1,    28,    -1,    28,    29,    -1,    26,    28,    29,    -1,
+      -1,    18,    -1,     3,    27,    22,    -1,    30,    -1,    29,
+      30,    -1,     5,    27,    22,    -1,     6,    27,    22,    -1,
+       7,    27,    22,    -1,     8,    27,    19,    -1,     8,    27,
+      20,    -1,     8,    27,    21,    -1,     9,    27,    22,    -1,
+      10,    27,    22,    -1,    11,    27,    22,    -1,    13,    27,
+      22,    -1,    14,    27,    22,    -1,    15,    17,    27,    22,
+      -1,    15,    16,    27,    22,    -1,    12,    27,    22,    -1,
+       4,    -1
 };
 
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const unsigned char yyrline[] =
 {
-       0,    67,    67,    68,    69,    70,    73,    74,    75,    79,
-      80,    83,    91,    92,    96,    97,    98,    99,   100,   101,
-     102,   103,   104,   105
+       0,    77,    77,    78,    79,    80,    83,    84,    85,    89,
+      90,    93,   101,   102,   106,   107,   108,   109,   110,   111,
+     112,   113,   114,   115,   116,   117,   118,   119,   120
 };
 #endif
 
@@ -360,30 +392,32 @@ static const unsigned char yyrline[] =
    First, the terminals, then, starting at YYNTOKENS, nonterminals. */
 static const char *const yytname[] =
 {
-  "$end", "error", "$undefined", "IDENTITY", "DEFAULT", "HOSTNAME", 
-  "USERNAME", "PASSWORD", "STARTTLS", "CERTIFICATE_PASSPHRASE", 
-  "PRECONNECT", "MDA", "MAP", "DISABLED", "ENABLED", "REQUIRED", "STRING", 
-  "NUMBER", "$accept", "rcfile", "identity_list", "map", "identity", 
-  "statement_list", "statement", 0
+  "$end", "error", "$undefined", "IDENTITY", "DEFAULT", "HOSTNAME",
+  "USERNAME", "PASSWORD", "STARTTLS", "CERTIFICATE_PASSPHRASE",
+  "PRECONNECT", "POSTCONNECT", "MDA", "QUALIFYDOMAIN", "HELO", "FORCE",
+  "SENDER", "REVERSE_PATH", "MAP", "DISABLED", "ENABLED", "REQUIRED",
+  "STRING", "NUMBER", "$accept", "rcfile", "identity_list", "map",
+  "identity", "statement_list", "statement", 0
 };
 #endif
 
 # ifdef YYPRINT
 /* YYTOKNUM[YYLEX-NUM] -- Internal token number corresponding to
    token YYLEX-NUM.  */
-static const unsigned short yytoknum[] =
+static const unsigned short int yytoknum[] =
 {
        0,   256,   257,   258,   259,   260,   261,   262,   263,   264,
-     265,   266,   267,   268,   269,   270,   271,   272
+     265,   266,   267,   268,   269,   270,   271,   272,   273,   274,
+     275,   276,   277,   278
 };
 # endif
 
 /* YYR1[YYN] -- Symbol number of symbol that rule YYN derives.  */
 static const unsigned char yyr1[] =
 {
-       0,    18,    19,    19,    19,    19,    20,    20,    20,    21,
-      21,    22,    23,    23,    24,    24,    24,    24,    24,    24,
-      24,    24,    24,    24
+       0,    24,    25,    25,    25,    25,    26,    26,    26,    27,
+      27,    28,    29,    29,    30,    30,    30,    30,    30,    30,
+      30,    30,    30,    30,    30,    30,    30,    30,    30
 };
 
 /* YYR2[YYN] -- Number of symbols composing right hand side of rule YYN.  */
@@ -391,7 +425,7 @@ static const unsigned char yyr2[] =
 {
        0,     2,     0,     1,     2,     1,     1,     2,     3,     0,
        1,     3,     1,     2,     3,     3,     3,     3,     3,     3,
-       3,     3,     3,     1
+       3,     3,     3,     3,     3,     4,     4,     3,     1
 };
 
 /* YYDEFACT[STATE-NAME] -- Default rule to reduce with in state
@@ -399,33 +433,37 @@ static const unsigned char yyr2[] =
    means the default is an error.  */
 static const unsigned char yydefact[] =
 {
-       2,     9,    23,     9,     9,     9,     9,     9,     9,     9,
-       0,     5,     6,     3,    12,    10,     0,     0,     0,     0,
-       0,     0,     0,     0,     1,     0,     7,     4,    13,    11,
-      14,    15,    16,    17,    18,    19,    20,    21,    22,     8
+       2,     9,    28,     9,     9,     9,     9,     9,     9,     9,
+       9,     9,     9,     0,     0,     5,     6,     3,    12,    10,
+       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
+       0,     9,     9,     1,     0,     7,     4,    13,    11,    14,
+      15,    16,    17,    18,    19,    20,    21,    22,    27,    23,
+      24,     0,     0,     8,    26,    25
 };
 
 /* YYDEFGOTO[NTERM-NUM]. */
 static const yysigned_char yydefgoto[] =
 {
-      -1,    10,    11,    16,    12,    13,    14
+      -1,    14,    15,    20,    16,    17,    18
 };
 
 /* YYPACT[STATE-NUM] -- Index in YYTABLE of the portion describing
    STATE-NUM.  */
-#define YYPACT_NINF -14
+#define YYPACT_NINF -18
 static const yysigned_char yypact[] =
 {
-       1,     2,   -14,     2,     2,     2,     2,     2,     2,     2,
-      15,    -1,    14,     1,   -14,   -14,    22,    23,    25,    26,
-      21,    27,    28,    29,   -14,    14,    14,    -1,   -14,   -14,
-     -14,   -14,   -14,   -14,   -14,   -14,   -14,   -14,   -14,    14
+      34,    -6,   -18,    -6,    -6,    -6,    -6,    -6,    -6,    -6,
+      -6,    -6,    -6,     3,    17,    18,    46,    34,   -18,   -18,
+       0,     1,     2,     4,    -5,     5,     6,    11,    12,    13,
+      40,    -6,    -6,   -18,    46,    46,    18,   -18,   -18,   -18,
+     -18,   -18,   -18,   -18,   -18,   -18,   -18,   -18,   -18,   -18,
+     -18,    41,    42,    46,   -18,   -18
 };
 
 /* YYPGOTO[NTERM-NUM].  */
 static const yysigned_char yypgoto[] =
 {
-     -14,   -14,    33,    24,   -10,    -9,   -13
+     -18,   -18,     8,    -2,    -4,    -3,   -17
 };
 
 /* YYTABLE[YYPACT[STATE-NUM]].  What to do in state STATE-NUM.  If
@@ -435,20 +473,24 @@ static const yysigned_char yypgoto[] =
 #define YYTABLE_NINF -1
 static const unsigned char yytable[] =
 {
-      28,    25,     1,    26,     1,     2,     3,     4,     5,     6,
-       7,     8,     9,    28,    15,    24,    39,    25,     2,     3,
-       4,     5,     6,     7,     8,     9,    28,    17,    18,    19,
-      20,    21,    22,    23,    33,    34,    35,     0,    29,    30,
-       0,    31,    32,    36,    37,    38,    27
+      37,    21,    22,    23,    24,    25,    26,    27,    28,    29,
+      30,    34,    19,    35,    42,    43,    44,    33,    37,    31,
+      32,     1,    38,    39,    40,    36,    41,    45,    46,    51,
+      52,    53,    34,    47,    48,    49,    37,     1,     2,     3,
+       4,     5,     6,     7,     8,     9,    10,    11,    12,    13,
+       2,     3,     4,     5,     6,     7,     8,     9,    10,    11,
+      12,    13,    50,    54,    55
 };
 
-static const yysigned_char yycheck[] =
+static const unsigned char yycheck[] =
 {
-      13,    11,     3,    12,     3,     4,     5,     6,     7,     8,
-       9,    10,    11,    26,    12,     0,    25,    27,     4,     5,
-       6,     7,     8,     9,    10,    11,    39,     3,     4,     5,
-       6,     7,     8,     9,    13,    14,    15,    -1,    16,    16,
-      -1,    16,    16,    16,    16,    16,    13
+      17,     3,     4,     5,     6,     7,     8,     9,    10,    11,
+      12,    15,    18,    16,    19,    20,    21,     0,    35,    16,
+      17,     3,    22,    22,    22,    17,    22,    22,    22,    31,
+      32,    34,    36,    22,    22,    22,    53,     3,     4,     5,
+       6,     7,     8,     9,    10,    11,    12,    13,    14,    15,
+       4,     5,     6,     7,     8,     9,    10,    11,    12,    13,
+      14,    15,    22,    22,    22
 };
 
 /* YYSTOS[STATE-NUM] -- The (internal number of the) accessing
@@ -456,9 +498,11 @@ static const yysigned_char yycheck[] =
 static const unsigned char yystos[] =
 {
        0,     3,     4,     5,     6,     7,     8,     9,    10,    11,
-      19,    20,    22,    23,    24,    12,    21,    21,    21,    21,
-      21,    21,    21,    21,     0,    22,    23,    20,    24,    16,
-      16,    16,    16,    13,    14,    15,    16,    16,    16,    23
+      12,    13,    14,    15,    25,    26,    28,    29,    30,    18,
+      27,    27,    27,    27,    27,    27,    27,    27,    27,    27,
+      27,    16,    17,     0,    28,    29,    26,    30,    22,    22,
+      22,    22,    19,    20,    21,    22,    22,    22,    22,    22,
+      22,    27,    27,    29,    22,    22
 };
 
 #if ! defined (YYSIZE_T) && defined (__SIZE_TYPE__)
@@ -484,7 +528,7 @@ static const unsigned char yystos[] =
 
 #define YYACCEPT	goto yyacceptlab
 #define YYABORT		goto yyabortlab
-#define YYERROR		goto yyerrlab1
+#define YYERROR		goto yyerrorlab
 
 
 /* Like YYERROR except do call yyerror.  This remains here temporarily
@@ -519,11 +563,11 @@ while (0)
    are run).  */
 
 #ifndef YYLLOC_DEFAULT
-# define YYLLOC_DEFAULT(Current, Rhs, N)         \
-  Current.first_line   = Rhs[1].first_line;      \
-  Current.first_column = Rhs[1].first_column;    \
-  Current.last_line    = Rhs[N].last_line;       \
-  Current.last_column  = Rhs[N].last_column;
+# define YYLLOC_DEFAULT(Current, Rhs, N)		\
+   ((Current).first_line   = (Rhs)[1].first_line,	\
+    (Current).first_column = (Rhs)[1].first_column,	\
+    (Current).last_line    = (Rhs)[N].last_line,	\
+    (Current).last_column  = (Rhs)[N].last_column)
 #endif
 
 /* YYLEX -- calling `yylex' with the right arguments.  */
@@ -567,17 +611,17 @@ do {								\
 
 /*------------------------------------------------------------------.
 | yy_stack_print -- Print the state stack from its BOTTOM up to its |
-| TOP (cinluded).                                                   |
+| TOP (included).                                                   |
 `------------------------------------------------------------------*/
 
 #if defined (__STDC__) || defined (__cplusplus)
 static void
-yy_stack_print (short *bottom, short *top)
+yy_stack_print (short int *bottom, short int *top)
 #else
 static void
 yy_stack_print (bottom, top)
-    short *bottom;
-    short *top;
+    short int *bottom;
+    short int *top;
 #endif
 {
   YYFPRINTF (stderr, "Stack now");
@@ -607,9 +651,9 @@ yy_reduce_print (yyrule)
 #endif
 {
   int yyi;
-  unsigned int yylineno = yyrline[yyrule];
+  unsigned int yylno = yyrline[yyrule];
   YYFPRINTF (stderr, "Reducing stack by rule %d (line %u), ",
-             yyrule - 1, yylineno);
+             yyrule - 1, yylno);
   /* Print the symbols being reduced, and their result.  */
   for (yyi = yyprhs[yyrule]; 0 <= yyrhs[yyi]; yyi++)
     YYFPRINTF (stderr, "%s ", yytname [yyrhs[yyi]]);
@@ -646,7 +690,7 @@ int yydebug;
    SIZE_MAX < YYSTACK_BYTES (YYMAXDEPTH)
    evaluated with infinite-precision integer arithmetic.  */
 
-#if YYMAXDEPTH == 0
+#if defined (YYMAXDEPTH) && YYMAXDEPTH == 0
 # undef YYMAXDEPTH
 #endif
 
@@ -844,9 +888,9 @@ yyparse ()
      to reallocate them elsewhere.  */
 
   /* The state stack.  */
-  short	yyssa[YYINITDEPTH];
-  short *yyss = yyssa;
-  register short *yyssp;
+  short int yyssa[YYINITDEPTH];
+  short int *yyss = yyssa;
+  register short int *yyssp;
 
   /* The semantic value stack.  */
   YYSTYPE yyvsa[YYINITDEPTH];
@@ -883,6 +927,7 @@ yyparse ()
   yyssp = yyss;
   yyvsp = yyvs;
 
+
   goto yysetstate;
 
 /*------------------------------------------------------------.
@@ -908,7 +953,7 @@ yyparse ()
 	   these so that the &'s don't force the real ones into
 	   memory.  */
 	YYSTYPE *yyvs1 = yyvs;
-	short *yyss1 = yyss;
+	short int *yyss1 = yyss;
 
 
 	/* Each stack pointer address is followed by the size of the
@@ -936,7 +981,7 @@ yyparse ()
 	yystacksize = YYMAXDEPTH;
 
       {
-	short *yyss1 = yyss;
+	short int *yyss1 = yyss;
 	union yyalloc *yyptr =
 	  (union yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
 	if (! yyptr)
@@ -1069,7 +1114,7 @@ yyreduce:
   switch (yyn)
     {
         case 11:
-#line 84 "parser.y"
+#line 94 "parser.y"
     {
 				identity = identity_new();
 				identity_add(identity);
@@ -1078,60 +1123,85 @@ yyreduce:
     break;
 
   case 14:
-#line 96 "parser.y"
+#line 106 "parser.y"
     { identity->host = xstrdup(yyvsp[0].sval); SET_DEFAULT_IDENTITY; }
     break;
 
   case 15:
-#line 97 "parser.y"
+#line 107 "parser.y"
     { identity->user = xstrdup(yyvsp[0].sval); SET_DEFAULT_IDENTITY; }
     break;
 
   case 16:
-#line 98 "parser.y"
+#line 108 "parser.y"
     { identity->pass = xstrdup(yyvsp[0].sval); SET_DEFAULT_IDENTITY; }
     break;
 
   case 17:
-#line 99 "parser.y"
+#line 109 "parser.y"
     { identity->starttls = Starttls_DISABLED; SET_DEFAULT_IDENTITY; }
     break;
 
   case 18:
-#line 100 "parser.y"
+#line 110 "parser.y"
     { identity->starttls = Starttls_ENABLED; SET_DEFAULT_IDENTITY; }
     break;
 
   case 19:
-#line 101 "parser.y"
+#line 111 "parser.y"
     { identity->starttls = Starttls_REQUIRED; SET_DEFAULT_IDENTITY; }
     break;
 
   case 20:
-#line 102 "parser.y"
+#line 112 "parser.y"
     { identity->certificate_passphrase = xstrdup(yyvsp[0].sval); SET_DEFAULT_IDENTITY; }
     break;
 
   case 21:
-#line 103 "parser.y"
+#line 113 "parser.y"
     { identity->preconnect = xstrdup(yyvsp[0].sval); SET_DEFAULT_IDENTITY; }
     break;
 
   case 22:
-#line 104 "parser.y"
-    { mda = xstrdup(yyvsp[0].sval); }
+#line 114 "parser.y"
+    { identity->postconnect = xstrdup(yyvsp[0].sval); SET_DEFAULT_IDENTITY; }
     break;
 
   case 23:
-#line 105 "parser.y"
+#line 115 "parser.y"
+    { identity->qualifydomain = xstrdup(yyvsp[0].sval); SET_DEFAULT_IDENTITY; }
+    break;
+
+  case 24:
+#line 116 "parser.y"
+    { identity->helo = xstrdup(yyvsp[0].sval); SET_DEFAULT_IDENTITY; }
+    break;
+
+  case 25:
+#line 117 "parser.y"
+    { identity->force_reverse_path = xstrdup(yyvsp[0].sval); SET_DEFAULT_IDENTITY; }
+    break;
+
+  case 26:
+#line 118 "parser.y"
+    { identity->force_sender = xstrdup(yyvsp[0].sval); SET_DEFAULT_IDENTITY; }
+    break;
+
+  case 27:
+#line 119 "parser.y"
+    { mda = xstrdup(yyvsp[0].sval); }
+    break;
+
+  case 28:
+#line 120 "parser.y"
     { default_identity = identity; }
     break;
 
 
     }
 
-/* Line 999 of yacc.c.  */
-#line 1135 "parser.c"
+/* Line 1010 of yacc.c.  */
+#line 1205 "parser.c"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -1172,18 +1242,33 @@ yyerrlab:
 	{
 	  YYSIZE_T yysize = 0;
 	  int yytype = YYTRANSLATE (yychar);
+	  const char* yyprefix;
 	  char *yymsg;
-	  int yyx, yycount;
+	  int yyx;
 
-	  yycount = 0;
 	  /* Start YYX at -YYN if negative to avoid negative indexes in
 	     YYCHECK.  */
-	  for (yyx = yyn < 0 ? -yyn : 0;
-	       yyx < (int) (sizeof (yytname) / sizeof (char *)); yyx++)
+	  int yyxbegin = yyn < 0 ? -yyn : 0;
+
+	  /* Stay within bounds of both yycheck and yytname.  */
+	  int yychecklim = YYLAST - yyn;
+	  int yyxend = yychecklim < YYNTOKENS ? yychecklim : YYNTOKENS;
+	  int yycount = 0;
+
+	  yyprefix = ", expecting ";
+	  for (yyx = yyxbegin; yyx < yyxend; ++yyx)
 	    if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
-	      yysize += yystrlen (yytname[yyx]) + 15, yycount++;
-	  yysize += yystrlen ("syntax error, unexpected ") + 1;
-	  yysize += yystrlen (yytname[yytype]);
+	      {
+		yysize += yystrlen (yyprefix) + yystrlen (yytname [yyx]);
+		yycount += 1;
+		if (yycount == 5)
+		  {
+		    yysize = 0;
+		    break;
+		  }
+	      }
+	  yysize += (sizeof ("syntax error, unexpected ")
+		     + yystrlen (yytname[yytype]));
 	  yymsg = (char *) YYSTACK_ALLOC (yysize);
 	  if (yymsg != 0)
 	    {
@@ -1192,16 +1277,13 @@ yyerrlab:
 
 	      if (yycount < 5)
 		{
-		  yycount = 0;
-		  for (yyx = yyn < 0 ? -yyn : 0;
-		       yyx < (int) (sizeof (yytname) / sizeof (char *));
-		       yyx++)
+		  yyprefix = ", expecting ";
+		  for (yyx = yyxbegin; yyx < yyxend; ++yyx)
 		    if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
 		      {
-			const char *yyq = ! yycount ? ", expecting " : " or ";
-			yyp = yystpcpy (yyp, yyq);
+			yyp = yystpcpy (yyp, yyprefix);
 			yyp = yystpcpy (yyp, yytname[yyx]);
-			yycount++;
+			yyprefix = " or ";
 		      }
 		}
 	      yyerror (yymsg);
@@ -1222,25 +1304,27 @@ yyerrlab:
       /* If just tried and failed to reuse lookahead token after an
 	 error, discard it.  */
 
-      /* Return failure if at end of input.  */
-      if (yychar == YYEOF)
+      if (yychar <= YYEOF)
         {
-	  /* Pop the error token.  */
-          YYPOPSTACK;
-	  /* Pop the rest of the stack.  */
-	  while (yyss < yyssp)
-	    {
-	      YYDSYMPRINTF ("Error: popping", yystos[*yyssp], yyvsp, yylsp);
-	      yydestruct (yystos[*yyssp], yyvsp);
-	      YYPOPSTACK;
-	    }
-	  YYABORT;
+          /* If at end of input, pop the error token,
+	     then the rest of the stack, then return failure.  */
+	  if (yychar == YYEOF)
+	     for (;;)
+	       {
+		 YYPOPSTACK;
+		 if (yyssp == yyss)
+		   YYABORT;
+		 YYDSYMPRINTF ("Error: popping", yystos[*yyssp], yyvsp, yylsp);
+		 yydestruct (yystos[*yyssp], yyvsp);
+	       }
         }
+      else
+	{
+	  YYDSYMPRINTF ("Error: discarding", yytoken, &yylval, &yylloc);
+	  yydestruct (yytoken, &yylval);
+	  yychar = YYEMPTY;
 
-      YYDSYMPRINTF ("Error: discarding", yytoken, &yylval, &yylloc);
-      yydestruct (yytoken, &yylval);
-      yychar = YYEMPTY;
-
+	}
     }
 
   /* Else will try to reuse lookahead token after shifting the error
@@ -1248,9 +1332,27 @@ yyerrlab:
   goto yyerrlab1;
 
 
-/*----------------------------------------------------.
-| yyerrlab1 -- error raised explicitly by an action.  |
-`----------------------------------------------------*/
+/*---------------------------------------------------.
+| yyerrorlab -- error raised explicitly by YYERROR.  |
+`---------------------------------------------------*/
+yyerrorlab:
+
+#ifdef __GNUC__
+  /* Pacify GCC when the user code never invokes YYERROR and the label
+     yyerrorlab therefore never appears in user code.  */
+  if (0)
+     goto yyerrorlab;
+#endif
+
+  yyvsp -= yylen;
+  yyssp -= yylen;
+  yystate = *yyssp;
+  goto yyerrlab1;
+
+
+/*-------------------------------------------------------------.
+| yyerrlab1 -- common code for both syntax error and YYERROR.  |
+`-------------------------------------------------------------*/
 yyerrlab1:
   yyerrstatus = 3;	/* Each real token shifted decrements this.  */
 
@@ -1274,9 +1376,8 @@ yyerrlab1:
 
       YYDSYMPRINTF ("Error: popping", yystos[*yyssp], yyvsp, yylsp);
       yydestruct (yystos[yystate], yyvsp);
-      yyvsp--;
-      yystate = *--yyssp;
-
+      YYPOPSTACK;
+      yystate = *yyssp;
       YY_STACK_PRINT (yyss, yyssp);
     }
 
@@ -1325,7 +1426,7 @@ yyreturn:
 }
 
 
-#line 108 "parser.y"
+#line 123 "parser.y"
 
 
 /* lexer interface */
@@ -1343,21 +1444,88 @@ void yyerror (const char *s)
 	exit(EX_CONFIG);
 }
 
+/**
+ * Check that a configuration file is secure.
+ * 
+ * \param securecheck if set to 1 strict file permission tests will be run.
+ * 
+ * \return 0 if everything is OK, -1 in case of error 
+ * 
+ */
+int rcfile_check(const char *pathname, const int securecheck)
+{
+#ifndef __EMX__
+	struct stat statbuf;
+
+	errno = 0;
+
+	/* special case useful for debugging purposes */
+	if (strcmp("/dev/null", pathname) == 0)
+		return 0;
+
+	/* pass through the special name for stdin */
+	if (strcmp("-", pathname) == 0)
+		return 0;
+
+	/* the run control file must have the same uid as the REAL uid of this 
+	 * process, it must have permissions no greater than 600, and it must
+	 * not be a symbolic link.  We check these conditions here. 
+	 */
+	if (lstat(pathname, &statbuf) < 0) {
+		if (errno == ENOENT) 
+			return 0;
+		else {
+			fprintf(stderr, "lstat: %s: %s\n", pathname, strerror(errno));
+			return -1;
+		}
+	}
+
+	if (!securecheck)
+		return 0;
+
+	if (!S_ISREG(statbuf.st_mode))
+	{
+		fprintf(stderr, "File %s must be a regular file.\n", pathname);
+		return -1;
+	}
+
+#ifndef __BEOS__
+	if (statbuf.st_mode & (S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH | S_IXOTH))
+	{
+		fprintf(stderr, "File %s must have no more than -rwx--x--- (0710) permissions.\n", pathname);
+		return -1;
+	}
+#endif /* !__BEOS__ */
+
+#ifdef HAVE_GETEUID
+	if (statbuf.st_uid != geteuid())
+#else
+	if (statbuf.st_uid != getuid())
+#endif /* HAVE_GETEUID */
+	{
+		fprintf(stderr, "File %s must be owned by you.\n", pathname);
+		return -1;
+	}
+#endif
+	return 0;
+}
+
+
 #define RCFILE "esmtprc"
 #define DOT_RCFILE "." RCFILE
-#define ETC_RCFILE "/etc/" RCFILE
+#define ETC_RCFILE SYSCONFDIR "/" RCFILE
 
 void rcfile_parse(const char *_rcfile)
 {
-	char *temp = NULL;
+	char *dot_rcfile = NULL;
 
 	if(_rcfile)
 	{
 		/* Configuration file specified on the command line */
-		if(!(yyin = fopen(_rcfile, "r")))
+		rcfile = _rcfile;
+		if(!(yyin = fopen(rcfile, "r")))
 			goto failure;
 			
-		rcfile = _rcfile;
 		goto success;
 	}
 	
@@ -1369,14 +1537,18 @@ void rcfile_parse(const char *_rcfile)
 		if (!(home = getenv("HOME")))
 			break;
 			
-		temp = xmalloc(strlen(home) + strlen(DOT_RCFILE) + 2);
+		dot_rcfile = xmalloc(strlen(home) + strlen(DOT_RCFILE) + 2);
 
-		strcpy(temp, home);
-		if (temp[strlen(temp) - 1] != '/')
-			strcat(temp, "/");
-		strcat(temp, DOT_RCFILE);
+		strcpy(dot_rcfile, home);
+		if (dot_rcfile[strlen(dot_rcfile) - 1] != '/')
+			strcat(dot_rcfile, "/");
+		strcat(dot_rcfile, DOT_RCFILE);
+		rcfile = dot_rcfile;
 
-		if(!(yyin = fopen(temp, "r")))
+ 		if (rcfile_check(rcfile, 1) < 0)
+ 			goto failure;
+ 
+		if(!(yyin = fopen(rcfile, "r")))
 		{
 			if(errno == ENOENT)
 				break;
@@ -1384,14 +1556,15 @@ void rcfile_parse(const char *_rcfile)
 				goto failure;
 		}
 		
-		rcfile = temp;
 		goto success;
 	} 
 	while(0);
 	
 	/* Search for the global configuration file */
-	do {
-		if(!(yyin = fopen(ETC_RCFILE, "r")))
+	do
+	{
+		rcfile = ETC_RCFILE;
+		if(!(yyin = fopen(rcfile, "r")))
 		{
 			if(errno == ENOENT)
 				break;
@@ -1399,13 +1572,13 @@ void rcfile_parse(const char *_rcfile)
 				goto failure;
 		}
 
-		rcfile = ETC_RCFILE;
 		goto success;
 	}
 	while(0);
 		
 	/* No configuration file found */
-	return;
+	fprintf(stderr, "No configuration file found at %s or " ETC_RCFILE "\n", dot_rcfile);
+	exit(EX_CONFIG);
 
 success:
 	/* Configuration file opened */
@@ -1420,8 +1593,8 @@ success:
 	fclose(yyin);	/* not checking this should be safe, file mode was r */
 
 	rcfile = NULL;
-	if(temp)
-		free(temp);
+	if(dot_rcfile)
+		free(dot_rcfile);
 
 	return;
 
